//*******************************************************
// GAA[V[ANZ[^M@ (accelerator_tx)
// (c)2007 Ryosuke Kawano
// URL: http://www.cts-net.ne.jp/~kawano-r/
// MCU: MC908QY4A with Serial Monitor
// Compiler: CodeWarrior HC08 V5.1 Special Edition
// 9.Apr.2007  Ver.1.0  GLWbNNo.2Ŕ\
//*******************************************************

#include <hidef.h>
#include <startup.h>
#include "derivative.h"

typedef unsigned char   U1;
typedef unsigned int    U2;
typedef signed int      S2;

//*************************
// }N̒`
//*************************

// M
#define PORT_TXOUT1     PTB_PTB6
#define PORT_TXOUT2     PTB_PTB7

#define TXOUT_ON        ( 1 )
#define TXOUT_OFF       ( 0 )

#define LEVEL1_TIME     ( ( U1 )( 40. / 10. ) )
#define LEVEL2_TIME     ( ( U1 )( 80. / 10. ) )
#define TX_BIT_TIME     ( ( U1 )( 120. / 10. ) )

// p[ԏ
#define INIT_TIME       ( ( U1 )( 500. / 10. ) )
#define POWER_OFF_TIME  ( ( U2 )( 60. * 1000. / 10. ) )

// ͏
#define PORT_REVERSE_SW    PTA_PTA2
#define PORT_SENSOR_POWER  PTA_PTA5

#define SW_ON           ( 0 )
#define SW_OFF          ( 1 )

// ANZ
#define HYSTERESIS      ( 12 )

// LEDo͏
#define PORT_REV_LED    PTA_PTA1
#define PORT_FWD1_LED   PTB_PTB0
#define PORT_FWD2_LED   PTB_PTB1
#define PORT_FWD3_LED   PTB_PTB2

#define LED_ON          ( 1 )
#define LED_OFF         ( 0 )

//*************************
// f[^̒`
//*************************

// S
volatile U1 U1_main_timing;

// p[ԏ
U1 U1_init_finish;

// ͏
U1 U1_addata;
U1 U1_reverse_sw;

// ANZ
U1 U1_accel;
const U1 U1_accel_table[ 3 ] = {
    0xE0,   // Accel Level 1
    0xA0,   // Accel Level 2
    0x70    // Accel Level 3
};

// ANZo[X
U1 U1_accel_reverse;
U1 U1_acc_rev_sleep_ready;

//*************************
// ֐vg^Cv錾
//*************************

void init_hardware( void );
void init_PORT( void );
void init_ADC( void );
void init_TIM( void );

void transmit( void );          // M
void power_mode( void );        // p[ԏ
void input( void );	            // ͏
void accel( void );	            // ANZ
void accel_reverse( void );	    // ANZo[X
void led_output( void );        // LEDo͏

//*************************
// C
//*************************

void main( void ) {
    init_hardware( );
    init_PORT( );
    init_ADC( );
    init_TIM( );
    
    EnableInterrupts;
    
    for ( ; ; ) {
        while ( U1_main_timing == 0 ){
            __RESET_WATCHDOG( );
        }
        U1_main_timing = 0;
        
        transmit( );            // M
        
        power_mode( );          // p[ԏ
        
        input( );               // ͏
        
        accel( );	            // ANZ
        
        accel_reverse( );	    // ANZo[X

        led_output( );          // LEDo͏
    }
}

//*************************
// Zbg̏
//*************************

void init_hardware( void ) {
    CONFIG2 = 0x00;     // IRQs֎~
    CONFIG1 = 0x02;     // STOPLACOPL
    OSCTRIM = Optional; 
}

void init_PORT( void ) {
    PTAPUE = 0x04;
    // PTA_PTA0, DDRA_DDRA0 ̓ANZX֎~
    PTA_PTA1 = 0;
    PTA_PTA3 = 0;
    PTA_PTA4 = 0;
    PTA_PTA5 = 0;
    DDRA_DDRA1 = 1;
    DDRA_DDRA3 = 1;
    DDRA_DDRA4 = 0;
    DDRA_DDRA5 = 1;
    PTBPUE = 0x00;
    PTB    = 0x00;
    DDRB   = 0xFF;
}

void init_ADC( void ) {
    ADCLK = 0x50;   // 3.2  4 = 0.8MHz ( 0.4 < fADCK < 2MHz )
    ADSCR = 0x22;
}

void init_TIM( void ) {
    TSC = 0x30;         // ^C}~A^C}ZbgAȂ
    TMOD = 32000 - 1;   // 0.3125s ~ 32000 = 10ms
    TSC = 0x40;         // TOF 荞݋A^C}X^[g
}

//*************************
// M
//*************************
void transmit( void ){
static U1 U1s_timer;
static enum {
    STATE_WAIT,
    STATE_ACCEL_ON,
    STATE_ACCEL_OFF,
    STATE_REVERSE
} ens_tx_state;

    switch ( ens_tx_state ){
    case STATE_WAIT:
        PORT_TXOUT1 = 0;
        PORT_TXOUT2 = 0;
        if ( U1_init_finish == 1 ){
            // yPz
            if ( ( U1_accel_reverse > 0 )
              && ( U1_accel_reverse < 4 ) ){
                U1s_timer = 0;
                PORT_TXOUT1 = 1;
                ens_tx_state = STATE_ACCEL_ON;
            } else {
                // yQz
                if ( U1_accel_reverse == 4 ){
                    PORT_TXOUT2 = 1;
                    ens_tx_state = STATE_REVERSE;
                }
            }
        }
        break;
    case STATE_ACCEL_ON:
        if ( U1s_timer < TX_BIT_TIME ){
            U1s_timer++;
        }
        // yRz
        if ( ( ( U1s_timer == LEVEL1_TIME )
            && ( U1_accel_reverse < 2 ) )
          || ( ( U1s_timer == LEVEL2_TIME )
            && ( U1_accel_reverse < 3 ) ) ){
            PORT_TXOUT1 = 0;
            ens_tx_state = STATE_ACCEL_OFF;
        } else {
            // ySz
            if ( ( U1s_timer >= TX_BIT_TIME )
              && ( U1_accel_reverse < 3 ) ){
                U1s_timer = LEVEL2_TIME;
                PORT_TXOUT1 = 0;
                ens_tx_state = STATE_ACCEL_OFF;
            } else {
                // yTz
                if ( U1_accel_reverse == 4 ){
                    PORT_TXOUT1 = 0;
                    PORT_TXOUT2 = 1;
                    ens_tx_state = STATE_REVERSE;
                }
            }
        }
        break;
    case STATE_ACCEL_OFF:
        if ( U1s_timer < TX_BIT_TIME ){
            U1s_timer++;
        }
        // yUz
        if ( U1s_timer >= ( TX_BIT_TIME - 1 ) ){
            ens_tx_state = STATE_WAIT;
        } else {
            // yVz
            if ( U1_accel_reverse == 4 ){
                PORT_TXOUT2 = 1;
                ens_tx_state = STATE_REVERSE;
            }
        }
        break;
    case STATE_REVERSE:
        // yWz
        if ( U1_accel_reverse < 4 ){
            PORT_TXOUT2 = 0;
            ens_tx_state = STATE_WAIT;
        }
        break;
    default:
        ens_tx_state = STATE_WAIT;
        break;             
    }
}

//*************************
// p[ԏ
//*************************
void power_mode( void ){
static U1 U1s_init_timer;
static U2 U2s_power_timer;

    PORT_SENSOR_POWER = 1;

    if ( U1_init_finish == 0 ){
        U1s_init_timer++;
        if ( U1s_init_timer >= INIT_TIME ){
            U1_init_finish = 1;
        }
    }

    if ( U2s_power_timer < POWER_OFF_TIME ){
        U2s_power_timer++;
        if ( U1_acc_rev_sleep_ready == 0 ){
            U2s_power_timer = 0;
        }
    } else {
        PTA = 0x00;
        PTB = 0x00;
        KBIER_KBIE2 = 1;    // POWER ON SW ݒ
        KBSCR_ACKK = 1;
        KBSCR_IMASKK = 0;
        asm( "STOP" );
        // STOP
        KBSCR_IMASKK = 1;
        for ( ; ; ){
            ;       // [vWDTZbg
        }
    }
}

//*************************
// ͏
//*************************
void input( void ){
static U1 U1s_count;
static U2 U2s_addata_sum;

    U2s_addata_sum += ADRL;
    U1s_count++;
    if ( U1s_count >= 2 ){
        U1_addata = ( U1 )( U2s_addata_sum / 2 );
        U2s_addata_sum = 0;
        U1s_count = 0;
    }
        
    U1_reverse_sw = 0;
    if ( PORT_REVERSE_SW == SW_ON ){
        U1_reverse_sw = 1;
    }
}

//*************************
// ANZ
//*************************
void accel( void ){
U1 U1t_temp;
    if ( U1_accel < 3 ){
        for ( U1t_temp = U1_accel ;
        ( ( U1t_temp < 3 )
        && ( U1_addata < ( U1_accel_table[ U1t_temp ] - HYSTERESIS ) ) ) ;
        U1t_temp++ ){
            U1_accel++;
        }
    }
    if ( U1_accel > 0 ){
        for ( U1t_temp = U1_accel ;
        ( ( U1t_temp > 0 )
        && ( U1_addata > ( U1_accel_table[ U1t_temp - 1 ] + HYSTERESIS ) ) ) ;
        U1t_temp-- ){
            U1_accel--;
        }
    }
}

//*************************
// ANZo[X
//*************************
void accel_reverse( void ){
    U1_accel_reverse = U1_accel;
    if ( U1_reverse_sw == 1 ){
        U1_accel_reverse = 4;
    }
    
    U1_acc_rev_sleep_ready = 0;
    if ( U1_accel_reverse == 0 ){   // ~̂Ƃ
        U1_acc_rev_sleep_ready = 1; // X[v
    }
}

//*************************
// LEDo͏
//*************************
void led_output( void ){
    // UASĂڂɂ͕ȂB
    PORT_FWD1_LED = 0;
    PORT_FWD2_LED = 0;
    PORT_FWD3_LED = 0;
    PORT_REV_LED = 0;

    if ( U1_init_finish == 1 ){
        switch( U1_accel_reverse ){
        case 1:
            PORT_FWD1_LED = 1;
            break;
        case 2:
            PORT_FWD2_LED = 1;
            break;
        case 3:
            PORT_FWD3_LED = 1;
            break;
        case 4:
            PORT_REV_LED = 1;
            break;
        default:
            break;
        }
    }
}
//*************************
// 荞ݏ
//*************************

// A to D Converter
#pragma TRAP_PROC
void _ADC_Interrupt( void ) {
}

// Keyboard Wakeup
#pragma TRAP_PROC
void _KBD_Interrupt( void ) {
    // ȂASTOPɗp
}

// Timer Overflow
#pragma TRAP_PROC
void _TOF_Interrupt( void ) {
    TSC_TOF = 0;
    // 10ms ɃC^C~Oʒm
    U1_main_timing = 1;
}

// Timer Channel 1
#pragma TRAP_PROC
void _TCH1_Interrupt( void ) {
//    TSC1_CH1F = 0;
}

// Timer Channel 0
#pragma TRAP_PROC
void _TCH0_Interrupt( void ) {
}

// IRQ
#pragma TRAP_PROC
void _IRQ_Interrupt( void ) {
}

// End Of File
